import { ApiJwtValidatePolicy, ApiCorsPolicy, ApiIpFilterPolicy } from "../models/apiManagement";
/**
 * APIM Policy build that can be used to build polices for APIs, Backends & operations
 */
export declare class ApimPolicyBuilder {
    private policyRoot;
    constructor();
    /**
     * Parses an APIM xml policy and loads it into a Policy Bulder
     * @param xml The xml policy
     */
    static parse(xml: string): Promise<ApimPolicyBuilder>;
    /**
     * Generates the XML policy that is compatible with APIM
     */
    build(): string;
    /**
     * Adds a policy to specify the APIM backend to use for an operation
     * @param backendId The APIM backend id
     */
    setBackendService(backendId: string): ApimPolicyBuilder;
    /**
     * Applies the CORS policy to an APIM API
     * @param corsConfig The APIM CORS configuration
     */
    cors(corsConfig: ApiCorsPolicy): ApimPolicyBuilder;
    /**
     * Applies an IP filter configuration policy to an APIM API
     * @param ipConfig The IP configuration policy
     */
    ipFilter(ipConfig: ApiIpFilterPolicy): ApimPolicyBuilder;
    /**
     * Applies the JWT validation policy to an APIM API
     * @param jwtPolicy The JWT validation configuration
     */
    jwtValidate(jwtPolicy: ApiJwtValidatePolicy): ApimPolicyBuilder;
    /**
     * Initializes an empty APIM policy
     */
    private init;
    /**
     * Create a claim element for a JWT validation policy
     * @param claim The JWT claim
     */
    private createClaimElement;
}
